@@ -5,16 +5,18 @@ from django.core.management.base import BaseCommand
 from django.conf import settings
 from django.contrib.staticfiles import finders
 from django.template.loader import render_to_string
+from django.utils.translation import to_locale
 
 
 class Command(BaseCommand):
 
     def handle(self, *args, **options):
         for code, name in settings.LANGUAGES:
+            code = to_locale(code)
             print "Processing", name
             path = finders.find('storage/src/locale/{code}.json'.format(code=code))
             if not path:
-                print "No file at", path, "Skipping"
+                print "No file for", code, "Skipping"
             else:
                 with io.open(path, "r", encoding="utf-8") as f:
                     print "Found file", path
@@ -32,4 +34,4 @@ class Command(BaseCommand):
                 "locale_code": code
             })
             print "Exporting to", path
-            f.write(content)
\ No newline at end of file
+            f.write(content)
